---
title: llarp::Crypto
summary: library crypto configuration 

---

# llarp::Crypto



library crypto configuration 


`#include <crypto.hpp>`

Inherited by [llarp::sodium::CryptoLibSodium](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md)

## Public Functions

|                | Name           |
| -------------- | -------------- |
| virtual | **[~Crypto](Classes/structllarp_1_1Crypto.md#function-~crypto)**() =0 |
| virtual std::optional< [AlignedBuffer](Classes/structllarp_1_1AlignedBuffer.md)< 32 > > | **[maybe_decrypt_name](Classes/structllarp_1_1Crypto.md#function-maybe-decrypt-name)**(std::string_view ciphertext, [SymmNonce](Classes/structllarp_1_1AlignedBuffer.md) nounce, std::string_view name) =0<br>decrypt cipherText name given the key generated from name  |
| virtual bool | **[xchacha20](Classes/structllarp_1_1Crypto.md#function-xchacha20)**(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & , const [SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [TunnelNonce](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>xchacha symmetric cipher  |
| virtual bool | **[xchacha20_alt](Classes/structllarp_1_1Crypto.md#function-xchacha20-alt)**(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & , const [SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const byte_t * ) =0<br>xchacha symmetric cipher (multibuffer)  |
| virtual bool | **[dh_client](Classes/structllarp_1_1Crypto.md#function-dh-client)**([SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [PubKey](Classes/structllarp_1_1PubKey.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [TunnelNonce](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>path dh creator's side  |
| virtual bool | **[dh_server](Classes/structllarp_1_1Crypto.md#function-dh-server)**([SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [PubKey](Classes/structllarp_1_1PubKey.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [TunnelNonce](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>path dh relay side  |
| virtual bool | **[transport_dh_client](Classes/structllarp_1_1Crypto.md#function-transport-dh-client)**([SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [PubKey](Classes/structllarp_1_1PubKey.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [TunnelNonce](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>transport dh client side  |
| virtual bool | **[transport_dh_server](Classes/structllarp_1_1Crypto.md#function-transport-dh-server)**([SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [PubKey](Classes/structllarp_1_1PubKey.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [TunnelNonce](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>transport dh server side  |
| virtual bool | **[shorthash](Classes/structllarp_1_1Crypto.md#function-shorthash)**([ShortHash](Classes/structllarp_1_1AlignedBuffer.md) & , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & ) =0<br>blake2b 256 bit  |
| virtual bool | **[hmac](Classes/structllarp_1_1Crypto.md#function-hmac)**(byte_t * , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & , const [SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>blake2s 256 bit "hmac" (keyed hash)  |
| virtual bool | **[sign](Classes/structllarp_1_1Crypto.md#function-sign)**([Signature](Classes/structllarp_1_1Signature.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & ) =0<br>ed25519 sign  |
| virtual bool | **[sign](Classes/structllarp_1_1Crypto.md#function-sign)**([Signature](Classes/structllarp_1_1Signature.md) & , const [PrivateKey](Classes/structllarp_1_1PrivateKey.md) & , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & ) =0<br>ed25519 sign (custom with derived keys)  |
| virtual bool | **[verify](Classes/structllarp_1_1Crypto.md#function-verify)**(const [PubKey](Classes/structllarp_1_1PubKey.md) & , const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & , const [Signature](Classes/structllarp_1_1Signature.md) & ) =0<br>ed25519 verify  |
| virtual bool | **[derive_subkey](Classes/structllarp_1_1Crypto.md#function-derive-subkey)**([PubKey](Classes/structllarp_1_1PubKey.md) & , const [PubKey](Classes/structllarp_1_1PubKey.md) & , uint64_t , const [AlignedBuffer](Classes/structllarp_1_1AlignedBuffer.md)< 32 > *  =nullptr) =0<br>derive sub keys for public keys  |
| virtual bool | **[derive_subkey_private](Classes/structllarp_1_1Crypto.md#function-derive-subkey-private)**([PrivateKey](Classes/structllarp_1_1PrivateKey.md) & , const [SecretKey](Classes/structllarp_1_1SecretKey.md) & , uint64_t , const [AlignedBuffer](Classes/structllarp_1_1AlignedBuffer.md)< 32 > *  =nullptr) =0<br>derive sub keys for private keys  |
| virtual bool | **[seed_to_secretkey](Classes/structllarp_1_1Crypto.md#function-seed-to-secretkey)**([llarp::SecretKey](Classes/structllarp_1_1SecretKey.md) & , const [llarp::IdentitySecret](Classes/structllarp_1_1IdentitySecret.md) & ) =0<br>seed to secretkey  |
| virtual void | **[randomize](Classes/structllarp_1_1Crypto.md#function-randomize)**(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & ) =0<br>randomize buffer  |
| virtual void | **[randbytes](Classes/structllarp_1_1Crypto.md#function-randbytes)**(byte_t * , size_t ) =0<br>randomizer memory  |
| virtual void | **[identity_keygen](Classes/structllarp_1_1Crypto.md#function-identity-keygen)**([SecretKey](Classes/structllarp_1_1SecretKey.md) & ) =0<br>generate signing keypair  |
| virtual void | **[encryption_keygen](Classes/structllarp_1_1Crypto.md#function-encryption-keygen)**([SecretKey](Classes/structllarp_1_1SecretKey.md) & ) =0<br>generate encryption keypair  |
| virtual void | **[pqe_keygen](Classes/structllarp_1_1Crypto.md#function-pqe-keygen)**([PQKeyPair](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>generate post quantum encrytion key  |
| virtual bool | **[pqe_decrypt](Classes/structllarp_1_1Crypto.md#function-pqe-decrypt)**(const [PQCipherBlock](Classes/structllarp_1_1AlignedBuffer.md) & , [SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const byte_t * ) =0<br>post quantum decrypt (buffer, sharedkey_dst, sec)  |
| virtual bool | **[pqe_encrypt](Classes/structllarp_1_1Crypto.md#function-pqe-encrypt)**([PQCipherBlock](Classes/structllarp_1_1AlignedBuffer.md) & , [SharedSecret](Classes/structllarp_1_1AlignedBuffer.md) & , const [PQPubKey](Classes/structllarp_1_1AlignedBuffer.md) & ) =0<br>post quantum encrypt (buffer, sharedkey_dst, pub)  |
| virtual bool | **[check_identity_privkey](Classes/structllarp_1_1Crypto.md#function-check-identity-privkey)**(const [SecretKey](Classes/structllarp_1_1SecretKey.md) & ) =0 |
| virtual bool | **[check_passwd_hash](Classes/structllarp_1_1Crypto.md#function-check-passwd-hash)**(std::string pwhash, std::string challenge) =0<br>check if a password hash string matches the challenge  |

## Public Functions Documentation

### function ~Crypto

```cpp
inline virtual ~Crypto() =0
```


### function maybe_decrypt_name

```cpp
virtual std::optional< AlignedBuffer< 32 > > maybe_decrypt_name(
    std::string_view ciphertext,
    SymmNonce nounce,
    std::string_view name
) =0
```

decrypt cipherText name given the key generated from name 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::maybe_decrypt_name](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-maybe-decrypt-name)


### function xchacha20

```cpp
virtual bool xchacha20(
    const llarp_buffer_t & ,
    const SharedSecret & ,
    const TunnelNonce & 
) =0
```

xchacha symmetric cipher 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::xchacha20](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-xchacha20)


### function xchacha20_alt

```cpp
virtual bool xchacha20_alt(
    const llarp_buffer_t & ,
    const llarp_buffer_t & ,
    const SharedSecret & ,
    const byte_t * 
) =0
```

xchacha symmetric cipher (multibuffer) 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::xchacha20_alt](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-xchacha20-alt)


### function dh_client

```cpp
virtual bool dh_client(
    SharedSecret & ,
    const PubKey & ,
    const SecretKey & ,
    const TunnelNonce & 
) =0
```

path dh creator's side 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::dh_client](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-dh-client)


### function dh_server

```cpp
virtual bool dh_server(
    SharedSecret & ,
    const PubKey & ,
    const SecretKey & ,
    const TunnelNonce & 
) =0
```

path dh relay side 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::dh_server](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-dh-server)


### function transport_dh_client

```cpp
virtual bool transport_dh_client(
    SharedSecret & ,
    const PubKey & ,
    const SecretKey & ,
    const TunnelNonce & 
) =0
```

transport dh client side 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::transport_dh_client](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-transport-dh-client)


### function transport_dh_server

```cpp
virtual bool transport_dh_server(
    SharedSecret & ,
    const PubKey & ,
    const SecretKey & ,
    const TunnelNonce & 
) =0
```

transport dh server side 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::transport_dh_server](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-transport-dh-server)


### function shorthash

```cpp
virtual bool shorthash(
    ShortHash & ,
    const llarp_buffer_t & 
) =0
```

blake2b 256 bit 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::shorthash](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-shorthash)


### function hmac

```cpp
virtual bool hmac(
    byte_t * ,
    const llarp_buffer_t & ,
    const SharedSecret & 
) =0
```

blake2s 256 bit "hmac" (keyed hash) 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::hmac](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-hmac)


### function sign

```cpp
virtual bool sign(
    Signature & ,
    const SecretKey & ,
    const llarp_buffer_t & 
) =0
```

ed25519 sign 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::sign](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-sign)


### function sign

```cpp
virtual bool sign(
    Signature & ,
    const PrivateKey & ,
    const llarp_buffer_t & 
) =0
```

ed25519 sign (custom with derived keys) 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::sign](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-sign)


### function verify

```cpp
virtual bool verify(
    const PubKey & ,
    const llarp_buffer_t & ,
    const Signature & 
) =0
```

ed25519 verify 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::verify](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-verify)


### function derive_subkey

```cpp
virtual bool derive_subkey(
    PubKey & ,
    const PubKey & ,
    uint64_t ,
    const AlignedBuffer< 32 > *  =nullptr
) =0
```

derive sub keys for public keys 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::derive_subkey](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-derive-subkey)


### function derive_subkey_private

```cpp
virtual bool derive_subkey_private(
    PrivateKey & ,
    const SecretKey & ,
    uint64_t ,
    const AlignedBuffer< 32 > *  =nullptr
) =0
```

derive sub keys for private keys 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::derive_subkey_private](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-derive-subkey-private)


### function seed_to_secretkey

```cpp
virtual bool seed_to_secretkey(
    llarp::SecretKey & ,
    const llarp::IdentitySecret & 
) =0
```

seed to secretkey 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::seed_to_secretkey](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-seed-to-secretkey)


### function randomize

```cpp
virtual void randomize(
    const llarp_buffer_t & 
) =0
```

randomize buffer 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::randomize](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-randomize)


### function randbytes

```cpp
virtual void randbytes(
    byte_t * ,
    size_t 
) =0
```

randomizer memory 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::randbytes](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-randbytes)


### function identity_keygen

```cpp
virtual void identity_keygen(
    SecretKey & 
) =0
```

generate signing keypair 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::identity_keygen](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-identity-keygen)


### function encryption_keygen

```cpp
virtual void encryption_keygen(
    SecretKey & 
) =0
```

generate encryption keypair 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::encryption_keygen](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-encryption-keygen)


### function pqe_keygen

```cpp
virtual void pqe_keygen(
    PQKeyPair & 
) =0
```

generate post quantum encrytion key 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::pqe_keygen](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-pqe-keygen)


### function pqe_decrypt

```cpp
virtual bool pqe_decrypt(
    const PQCipherBlock & ,
    SharedSecret & ,
    const byte_t * 
) =0
```

post quantum decrypt (buffer, sharedkey_dst, sec) 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::pqe_decrypt](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-pqe-decrypt)


### function pqe_encrypt

```cpp
virtual bool pqe_encrypt(
    PQCipherBlock & ,
    SharedSecret & ,
    const PQPubKey & 
) =0
```

post quantum encrypt (buffer, sharedkey_dst, pub) 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::pqe_encrypt](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-pqe-encrypt)


### function check_identity_privkey

```cpp
virtual bool check_identity_privkey(
    const SecretKey & 
) =0
```


**Reimplemented by**: [llarp::sodium::CryptoLibSodium::check_identity_privkey](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-check-identity-privkey)


### function check_passwd_hash

```cpp
virtual bool check_passwd_hash(
    std::string pwhash,
    std::string challenge
) =0
```

check if a password hash string matches the challenge 

**Reimplemented by**: [llarp::sodium::CryptoLibSodium::check_passwd_hash](Classes/structllarp_1_1sodium_1_1CryptoLibSodium.md#function-check-passwd-hash)


-------------------------------

Updated on 2023-01-31 at 18:48:37 +0000