---
title: llarp::exit::BaseSession
summary: a persisting exit session with an exit router 

---

# llarp::exit::BaseSession



a persisting exit session with an exit router 


`#include <session.hpp>`

Inherits from [llarp::path::Builder](Classes/structllarp_1_1path_1_1Builder.md), std::enable_shared_from_this< BaseSession >, [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)

Inherited by [llarp::exit::ExitSession](Classes/structllarp_1_1exit_1_1ExitSession.md), [llarp::exit::SNodeSession](Classes/structllarp_1_1exit_1_1SNodeSession.md)

## Public Functions

|                | Name           |
| -------------- | -------------- |
| | **[BaseSession](Classes/structllarp_1_1exit_1_1BaseSession.md#function-basesession)**(const [llarp::RouterID](Classes/structllarp_1_1RouterID.md) & exitRouter, std::function< bool(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) &)> writepkt, [AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * r, size_t numpaths, size_t hoplen, [EndpointBase](Classes/classllarp_1_1EndpointBase.md) * parent) |
| | **[~BaseSession](Classes/structllarp_1_1exit_1_1BaseSession.md#function-~basesession)**() override |
| virtual std::shared_ptr< [path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md) > | **[GetSelf](Classes/structllarp_1_1exit_1_1BaseSession.md#function-getself)**() override<br>get a shared_ptr of ourself  |
| virtual std::weak_ptr< [path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md) > | **[GetWeak](Classes/structllarp_1_1exit_1_1BaseSession.md#function-getweak)**() override<br>get a weak_ptr of ourself  |
| virtual void | **[BlacklistSNode](Classes/structllarp_1_1exit_1_1BaseSession.md#function-blacklistsnode)**(const [RouterID](Classes/structllarp_1_1RouterID.md) snode) override |
| util::StatusObject | **[ExtractStatus](Classes/structllarp_1_1exit_1_1BaseSession.md#function-extractstatus)**() const |
| virtual bool | **[ShouldBundleRC](Classes/structllarp_1_1exit_1_1BaseSession.md#function-shouldbundlerc)**() const override<br>should we bundle RCs in builds?  |
| virtual void | **[ResetInternalState](Classes/structllarp_1_1exit_1_1BaseSession.md#function-resetinternalstate)**() override<br>reset all cooldown timers  |
| virtual bool | **[UrgentBuild](Classes/structllarp_1_1exit_1_1BaseSession.md#function-urgentbuild)**(llarp_time_t ) const override |
| virtual void | **[HandlePathDied](Classes/structllarp_1_1exit_1_1BaseSession.md#function-handlepathdied)**(llarp::path::Path_ptr path) override<br>a path died now what?  |
| bool | **[CheckPathDead](Classes/structllarp_1_1exit_1_1BaseSession.md#function-checkpathdead)**(path::Path_ptr p, llarp_time_t dlt) |
| virtual std::optional< std::vector< [RouterContact](Classes/structllarp_1_1RouterContact.md) > > | **[GetHopsForBuild](Classes/structllarp_1_1exit_1_1BaseSession.md#function-gethopsforbuild)**() override |
| virtual bool | **[ShouldBuildMore](Classes/structllarp_1_1exit_1_1BaseSession.md#function-shouldbuildmore)**(llarp_time_t now) const override<br>return true if we should build another path  |
| virtual void | **[HandlePathBuilt](Classes/structllarp_1_1exit_1_1BaseSession.md#function-handlepathbuilt)**(llarp::path::Path_ptr p) override |
| bool | **[QueueUpstreamTraffic](Classes/structllarp_1_1exit_1_1BaseSession.md#function-queueupstreamtraffic)**([llarp::net::IPPacket](Classes/structllarp_1_1net_1_1IPPacket.md) pkt, const size_t packSize, service::ProtocolType t) |
| bool | **[FlushUpstream](Classes/structllarp_1_1exit_1_1BaseSession.md#function-flushupstream)**()<br>flush upstream to exit via paths  |
| void | **[FlushDownstream](Classes/structllarp_1_1exit_1_1BaseSession.md#function-flushdownstream)**()<br>flush downstream to user via tun  |
| virtual path::PathRole | **[GetRoles](Classes/structllarp_1_1exit_1_1BaseSession.md#function-getroles)**() const override<br>get roles for this path builder  |
| virtual bool | **[Stop](Classes/structllarp_1_1exit_1_1BaseSession.md#function-stop)**() override<br>send close and stop session  |
| bool | **[IsReady](Classes/structllarp_1_1exit_1_1BaseSession.md#function-isready)**() const |
| const [llarp::RouterID](Classes/structllarp_1_1RouterID.md) | **[Endpoint](Classes/structllarp_1_1exit_1_1BaseSession.md#function-endpoint)**() const |
| std::optional< [PathID_t](Classes/structllarp_1_1PathID__t.md) > | **[CurrentPath](Classes/structllarp_1_1exit_1_1BaseSession.md#function-currentpath)**() const |
| bool | **[IsExpired](Classes/structllarp_1_1exit_1_1BaseSession.md#function-isexpired)**(llarp_time_t now) const |
| bool | **[LoadIdentityFromFile](Classes/structllarp_1_1exit_1_1BaseSession.md#function-loadidentityfromfile)**(const char * fname) |
| void | **[AddReadyHook](Classes/structllarp_1_1exit_1_1BaseSession.md#function-addreadyhook)**(SessionReadyFunc func) |

## Protected Functions

|                | Name           |
| -------------- | -------------- |
| virtual void | **[PopulateRequest](Classes/structllarp_1_1exit_1_1BaseSession.md#function-populaterequest)**([llarp::routing::ObtainExitMessage](Classes/structllarp_1_1routing_1_1ObtainExitMessage.md) & msg) const =0 |
| bool | **[HandleTrafficDrop](Classes/structllarp_1_1exit_1_1BaseSession.md#function-handletrafficdrop)**(llarp::path::Path_ptr p, const [llarp::PathID_t](Classes/structllarp_1_1PathID__t.md) & path, uint64_t s) |
| bool | **[HandleGotExit](Classes/structllarp_1_1exit_1_1BaseSession.md#function-handlegotexit)**(llarp::path::Path_ptr p, llarp_time_t b) |
| bool | **[HandleTraffic](Classes/structllarp_1_1exit_1_1BaseSession.md#function-handletraffic)**(llarp::path::Path_ptr p, const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & buf, uint64_t seqno, service::ProtocolType t) |

## Public Attributes

|                | Name           |
| -------------- | -------------- |
| constexpr size_t | **[MaxUpstreamQueueLength](Classes/structllarp_1_1exit_1_1BaseSession.md#variable-maxupstreamqueuelength)**  |

## Protected Attributes

|                | Name           |
| -------------- | -------------- |
| [llarp::RouterID](Classes/structllarp_1_1RouterID.md) | **[m_ExitRouter](Classes/structllarp_1_1exit_1_1BaseSession.md#variable-m-exitrouter)**  |
| [llarp::SecretKey](Classes/structllarp_1_1SecretKey.md) | **[m_ExitIdentity](Classes/structllarp_1_1exit_1_1BaseSession.md#variable-m-exitidentity)**  |
| std::function< bool(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) &)> | **[m_WritePacket](Classes/structllarp_1_1exit_1_1BaseSession.md#variable-m-writepacket)**  |

## Additional inherited members

**Public Functions inherited from [llarp::path::Builder](Classes/structllarp_1_1path_1_1Builder.md)**

|                | Name           |
| -------------- | -------------- |
| | **[Builder](Classes/structllarp_1_1path_1_1Builder.md#function-builder)**([AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * p_router, size_t numDesiredPaths, size_t numHops)<br>construct  |
| virtual | **[~Builder](Classes/structllarp_1_1path_1_1Builder.md#function-~builder)**() =default |
| bool | **[BuildCooldownHit](Classes/structllarp_1_1path_1_1Builder.md#function-buildcooldownhit)**(llarp_time_t now) const<br>return true if we hit our soft limit for building paths too fast  |
| [BuildStats](Classes/structllarp_1_1path_1_1BuildStats.md) | **[CurrentBuildStats](Classes/structllarp_1_1path_1_1Builder.md#function-currentbuildstats)**() const |
| virtual bool | **[IsStopped](Classes/structllarp_1_1path_1_1Builder.md#function-isstopped)**() const override<br>return true if we are stopped  |
| virtual bool | **[ShouldRemove](Classes/structllarp_1_1path_1_1Builder.md#function-shouldremove)**() const override<br>return true if we can and should remove this pathset and underlying resources from its parent context  |
| virtual llarp_time_t | **[Now](Classes/structllarp_1_1path_1_1Builder.md#function-now)**() const override<br>get time from event loop  |
| virtual void | **[Tick](Classes/structllarp_1_1path_1_1Builder.md#function-tick)**(llarp_time_t now) override<br>tick owned paths  |
| virtual void | **[BuildOne](Classes/structllarp_1_1path_1_1Builder.md#function-buildone)**(PathRole roles =ePathRoleAny) override |
| virtual bool | **[BuildOneAlignedTo](Classes/structllarp_1_1path_1_1Builder.md#function-buildonealignedto)**(const [RouterID](Classes/structllarp_1_1RouterID.md) endpoint) override |
| std::optional< std::vector< [RouterContact](Classes/structllarp_1_1RouterContact.md) > > | **[GetHopsAlignedToForBuild](Classes/structllarp_1_1path_1_1Builder.md#function-gethopsalignedtoforbuild)**([RouterID](Classes/structllarp_1_1RouterID.md) endpoint, const std::set< [RouterID](Classes/structllarp_1_1RouterID.md) > & exclude ={}) |
| virtual void | **[Build](Classes/structllarp_1_1path_1_1Builder.md#function-build)**(std::vector< [RouterContact](Classes/structllarp_1_1RouterContact.md) > hops, PathRole roles =ePathRoleAny) override<br>manual build on these hops  |
| std::optional< [RouterContact](Classes/structllarp_1_1RouterContact.md) > | **[SelectFirstHop](Classes/structllarp_1_1path_1_1Builder.md#function-selectfirsthop)**(const std::set< [RouterID](Classes/structllarp_1_1RouterID.md) > & exclude ={}) const<br>pick a first hop  |
| void | **[ManualRebuild](Classes/structllarp_1_1path_1_1Builder.md#function-manualrebuild)**(size_t N, PathRole roles =ePathRoleAny) |
| virtual const [SecretKey](Classes/structllarp_1_1SecretKey.md) & | **[GetTunnelEncryptionSecretKey](Classes/structllarp_1_1path_1_1Builder.md#function-gettunnelencryptionsecretkey)**() const |
| virtual void | **[HandlePathBuildTimeout](Classes/structllarp_1_1path_1_1Builder.md#function-handlepathbuildtimeout)**(Path_ptr p) override |
| virtual void | **[HandlePathBuildFailedAt](Classes/structllarp_1_1path_1_1Builder.md#function-handlepathbuildfailedat)**(Path_ptr p, [RouterID](Classes/structllarp_1_1RouterID.md) hop) override |

**Protected Functions inherited from [llarp::path::Builder](Classes/structllarp_1_1path_1_1Builder.md)**

|                | Name           |
| -------------- | -------------- |
| bool | **[BuildCooldownHit](Classes/structllarp_1_1path_1_1Builder.md#function-buildcooldownhit)**([RouterID](Classes/structllarp_1_1RouterID.md) edge) const<br>return true if we hit our soft limit for building paths too fast on a first hop  |

**Public Attributes inherited from [llarp::path::Builder](Classes/structllarp_1_1path_1_1Builder.md)**

|                | Name           |
| -------------- | -------------- |
| [AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) *const | **[m_router](Classes/structllarp_1_1path_1_1Builder.md#variable-m-router)**  |
| [SecretKey](Classes/structllarp_1_1SecretKey.md) | **[enckey](Classes/structllarp_1_1path_1_1Builder.md#variable-enckey)**  |
| size_t | **[numHops](Classes/structllarp_1_1path_1_1Builder.md#variable-numhops)**  |
| llarp_time_t | **[lastBuild](Classes/structllarp_1_1path_1_1Builder.md#variable-lastbuild)**  |
| llarp_time_t | **[buildIntervalLimit](Classes/structllarp_1_1path_1_1Builder.md#variable-buildintervallimit)**  |

**Protected Attributes inherited from [llarp::path::Builder](Classes/structllarp_1_1path_1_1Builder.md)**

|                | Name           |
| -------------- | -------------- |
| std::atomic< bool > | **[_run](Classes/structllarp_1_1path_1_1Builder.md#variable--run)** <br>flag for [PathSet::Stop()]() |

**Protected Types inherited from [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)**

|                | Name           |
| -------------- | -------------- |
| using [util::NullMutex](Classes/structllarp_1_1util_1_1NullMutex.md) | **[Mtx_t](Classes/structllarp_1_1path_1_1PathSet.md#using-mtx-t)**  |
| using [util::NullLock](Classes/structllarp_1_1util_1_1NullLock.md) | **[Lock_t](Classes/structllarp_1_1path_1_1PathSet.md#using-lock-t)**  |
| using std::unordered_map< std::pair< [RouterID](Classes/structllarp_1_1RouterID.md), [PathID_t](Classes/structllarp_1_1PathID__t.md) >, Path_ptr > | **[PathMap_t](Classes/structllarp_1_1path_1_1PathSet.md#using-pathmap-t)**  |

**Public Functions inherited from [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)**

|                | Name           |
| -------------- | -------------- |
| | **[PathSet](Classes/structllarp_1_1path_1_1PathSet.md#function-pathset)**(size_t numDesiredPaths)<br>construct @params numDesiredPaths the number of paths to maintain  |
| virtual void | **[BuildOne](Classes/structllarp_1_1path_1_1PathSet.md#function-buildone)**(PathRole roles =ePathRoleAny) =0 |
| virtual void | **[Build](Classes/structllarp_1_1path_1_1PathSet.md#function-build)**(std::vector< [RouterContact](Classes/structllarp_1_1RouterContact.md) > hops, PathRole roles =ePathRoleAny) =0<br>manual build on these hops  |
| virtual void | **[Tick](Classes/structllarp_1_1path_1_1PathSet.md#function-tick)**(llarp_time_t now)<br>tick owned paths  |
| size_t | **[NumPathsExistingAt](Classes/structllarp_1_1path_1_1PathSet.md#function-numpathsexistingat)**(llarp_time_t futureTime) const<br>count the number of paths that will exist at this timestamp in future  |
| virtual void | **[HandlePathBuildTimeout](Classes/structllarp_1_1path_1_1PathSet.md#function-handlepathbuildtimeout)**(Path_ptr path) |
| virtual void | **[HandlePathBuildFailedAt](Classes/structllarp_1_1path_1_1PathSet.md#function-handlepathbuildfailedat)**(Path_ptr path, [RouterID](Classes/structllarp_1_1RouterID.md) hop) |
| virtual void | **[PathBuildStarted](Classes/structllarp_1_1path_1_1PathSet.md#function-pathbuildstarted)**(Path_ptr path) |
| bool | **[GetNewestIntro](Classes/structllarp_1_1path_1_1PathSet.md#function-getnewestintro)**([service::Introduction](Classes/structllarp_1_1service_1_1Introduction.md) & intro) const |
| void | **[AddPath](Classes/structllarp_1_1path_1_1PathSet.md#function-addpath)**(Path_ptr path) |
| Path_ptr | **[GetByUpstream](Classes/structllarp_1_1path_1_1PathSet.md#function-getbyupstream)**([RouterID](Classes/structllarp_1_1RouterID.md) remote, [PathID_t](Classes/structllarp_1_1PathID__t.md) rxid) const |
| void | **[ExpirePaths](Classes/structllarp_1_1path_1_1PathSet.md#function-expirepaths)**(llarp_time_t now, [AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * router) |
| size_t | **[NumInStatus](Classes/structllarp_1_1path_1_1PathSet.md#function-numinstatus)**(PathStatus st) const<br>get the number of paths in this status  |
| size_t | **[AvailablePaths](Classes/structllarp_1_1path_1_1PathSet.md#function-availablepaths)**(PathRole role) const<br>get the number of paths that match the role that are available  |
| virtual llarp_time_t | **[Now](Classes/structllarp_1_1path_1_1PathSet.md#function-now)**() const =0<br>get time from event loop  |
| virtual bool | **[IsStopped](Classes/structllarp_1_1path_1_1PathSet.md#function-isstopped)**() const =0<br>return true if we are stopped  |
| virtual std::string | **[Name](Classes/structllarp_1_1path_1_1PathSet.md#function-name)**() const =0<br>get the "name" of this pathset  |
| virtual bool | **[ShouldRemove](Classes/structllarp_1_1path_1_1PathSet.md#function-shouldremove)**() const =0<br>return true if we can and should remove this pathset and underlying resources from its parent context  |
| virtual bool | **[ShouldBuildMoreForRoles](Classes/structllarp_1_1path_1_1PathSet.md#function-shouldbuildmoreforroles)**(llarp_time_t now, PathRole roles) const<br>return true if we need another path with the given path roles  |
| virtual size_t | **[MinRequiredForRoles](Classes/structllarp_1_1path_1_1PathSet.md#function-minrequiredforroles)**(PathRole roles) const<br>return the minimum number of paths we want for given roles  |
| virtual bool | **[ShouldPublishDescriptors](Classes/structllarp_1_1path_1_1PathSet.md#function-shouldpublishdescriptors)**(] llarp_time_t now) const<br>return true if we should publish a new hidden service descriptor  |
| virtual bool | **[HandleGotIntroMessage](Classes/structllarp_1_1path_1_1PathSet.md#function-handlegotintromessage)**(std::shared_ptr< const [dht::GotIntroMessage](Classes/structllarp_1_1dht_1_1GotIntroMessage.md) > )<br>override me in subtype  |
| virtual bool | **[HandleGotRouterMessage](Classes/structllarp_1_1path_1_1PathSet.md#function-handlegotroutermessage)**(std::shared_ptr< const [dht::GotRouterMessage](Classes/structllarp_1_1dht_1_1GotRouterMessage.md) > )<br>override me in subtype  |
| virtual bool | **[HandleGotNameMessage](Classes/structllarp_1_1path_1_1PathSet.md#function-handlegotnamemessage)**(std::shared_ptr< const [dht::GotNameMessage](Classes/structllarp_1_1dht_1_1GotNameMessage.md) > )<br>override me in subtype  |
| virtual [routing::IMessageHandler](Classes/structllarp_1_1routing_1_1IMessageHandler.md) * | **[GetDHTHandler](Classes/structllarp_1_1path_1_1PathSet.md#function-getdhthandler)**() |
| Path_ptr | **[GetEstablishedPathClosestTo](Classes/structllarp_1_1path_1_1PathSet.md#function-getestablishedpathclosestto)**([RouterID](Classes/structllarp_1_1RouterID.md) router, std::unordered_set< [RouterID](Classes/structllarp_1_1RouterID.md) > excluding ={}, PathRole roles =ePathRoleAny) const |
| Path_ptr | **[PickEstablishedPath](Classes/structllarp_1_1path_1_1PathSet.md#function-pickestablishedpath)**(PathRole roles =ePathRoleAny) const |
| Path_ptr | **[PickRandomEstablishedPath](Classes/structllarp_1_1path_1_1PathSet.md#function-pickrandomestablishedpath)**(PathRole roles =ePathRoleAny) const |
| Path_ptr | **[GetPathByRouter](Classes/structllarp_1_1path_1_1PathSet.md#function-getpathbyrouter)**([RouterID](Classes/structllarp_1_1RouterID.md) router, PathRole roles =ePathRoleAny) const |
| Path_ptr | **[GetNewestPathByRouter](Classes/structllarp_1_1path_1_1PathSet.md#function-getnewestpathbyrouter)**([RouterID](Classes/structllarp_1_1RouterID.md) router, PathRole roles =ePathRoleAny) const |
| Path_ptr | **[GetRandomPathByRouter](Classes/structllarp_1_1path_1_1PathSet.md#function-getrandompathbyrouter)**([RouterID](Classes/structllarp_1_1RouterID.md) router, PathRole roles =ePathRoleAny) const |
| Path_ptr | **[GetPathByID](Classes/structllarp_1_1path_1_1PathSet.md#function-getpathbyid)**([PathID_t](Classes/structllarp_1_1PathID__t.md) id) const |
| Path_ptr | **[GetByEndpointWithID](Classes/structllarp_1_1path_1_1PathSet.md#function-getbyendpointwithid)**([RouterID](Classes/structllarp_1_1RouterID.md) router, [PathID_t](Classes/structllarp_1_1PathID__t.md) id) const |
| std::optional< std::set< [service::Introduction](Classes/structllarp_1_1service_1_1Introduction.md) > > | **[GetCurrentIntroductionsWithFilter](Classes/structllarp_1_1path_1_1PathSet.md#function-getcurrentintroductionswithfilter)**(std::function< bool(const [service::Introduction](Classes/structllarp_1_1service_1_1Introduction.md) &)> filter) const |
| virtual bool | **[PublishIntroSet](Classes/structllarp_1_1path_1_1PathSet.md#function-publishintroset)**(const [service::EncryptedIntroSet](Classes/structllarp_1_1service_1_1EncryptedIntroSet.md) & , [AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * ) |
| virtual bool | **[BuildOneAlignedTo](Classes/structllarp_1_1path_1_1PathSet.md#function-buildonealignedto)**(const [RouterID](Classes/structllarp_1_1RouterID.md) endpoint) =0 |
| virtual void | **[SendPacketToRemote](Classes/structllarp_1_1path_1_1PathSet.md#function-sendpackettoremote)**(const [llarp_buffer_t](Classes/structllarp__buffer__t.md) & pkt, service::ProtocolType t) =0 |
| void | **[ForEachPath](Classes/structllarp_1_1path_1_1PathSet.md#function-foreachpath)**(std::function< void(const Path_ptr &)> visit) const |
| void | **[UpstreamFlush](Classes/structllarp_1_1path_1_1PathSet.md#function-upstreamflush)**([AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * r) |
| void | **[DownstreamFlush](Classes/structllarp_1_1path_1_1PathSet.md#function-downstreamflush)**([AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * r) |

**Protected Functions inherited from [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)**

|                | Name           |
| -------------- | -------------- |
| void | **[TickPaths](Classes/structllarp_1_1path_1_1PathSet.md#function-tickpaths)**([AbstractRouter](Classes/structllarp_1_1AbstractRouter.md) * r) |

**Public Attributes inherited from [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)**

|                | Name           |
| -------------- | -------------- |
| constexpr size_t | **[max_paths](Classes/structllarp_1_1path_1_1PathSet.md#variable-max-paths)** <br>maximum number of paths a path set can maintain  |
| size_t | **[numDesiredPaths](Classes/structllarp_1_1path_1_1PathSet.md#variable-numdesiredpaths)**  |

**Protected Attributes inherited from [llarp::path::PathSet](Classes/structllarp_1_1path_1_1PathSet.md)**

|                | Name           |
| -------------- | -------------- |
| [BuildStats](Classes/structllarp_1_1path_1_1BuildStats.md) | **[m_BuildStats](Classes/structllarp_1_1path_1_1PathSet.md#variable-m-buildstats)**  |
| [Mtx_t](Classes/structllarp_1_1util_1_1NullMutex.md) | **[m_PathsMutex](Classes/structllarp_1_1path_1_1PathSet.md#variable-m-pathsmutex)**  |
| PathMap_t | **[m_Paths](Classes/structllarp_1_1path_1_1PathSet.md#variable-m-paths)**  |


## Public Functions Documentation

### function BaseSession

```cpp
BaseSession(
    const llarp::RouterID & exitRouter,
    std::function< bool(const llarp_buffer_t &)> writepkt,
    AbstractRouter * r,
    size_t numpaths,
    size_t hoplen,
    EndpointBase * parent
)
```


### function ~BaseSession

```cpp
~BaseSession() override
```


### function GetSelf

```cpp
inline virtual std::shared_ptr< path::PathSet > GetSelf() override
```

get a shared_ptr of ourself 

**Reimplements**: [llarp::path::PathSet::GetSelf](Classes/structllarp_1_1path_1_1PathSet.md#function-getself)


### function GetWeak

```cpp
inline virtual std::weak_ptr< path::PathSet > GetWeak() override
```

get a weak_ptr of ourself 

**Reimplements**: [llarp::path::PathSet::GetWeak](Classes/structllarp_1_1path_1_1PathSet.md#function-getweak)


### function BlacklistSNode

```cpp
virtual void BlacklistSNode(
    const RouterID snode
) override
```


**Reimplements**: [llarp::path::PathSet::BlacklistSNode](Classes/structllarp_1_1path_1_1PathSet.md#function-blacklistsnode)


### function ExtractStatus

```cpp
util::StatusObject ExtractStatus() const
```


### function ShouldBundleRC

```cpp
inline virtual bool ShouldBundleRC() const override
```

should we bundle RCs in builds? 

**Reimplements**: [llarp::path::Builder::ShouldBundleRC](Classes/structllarp_1_1path_1_1Builder.md#function-shouldbundlerc)


### function ResetInternalState

```cpp
virtual void ResetInternalState() override
```

reset all cooldown timers 

**Reimplements**: [llarp::path::PathSet::ResetInternalState](Classes/structllarp_1_1path_1_1PathSet.md#function-resetinternalstate)


### function UrgentBuild

```cpp
virtual bool UrgentBuild(
    llarp_time_t 
) const override
```


**Reimplements**: [llarp::path::Builder::UrgentBuild](Classes/structllarp_1_1path_1_1Builder.md#function-urgentbuild)


### function HandlePathDied

```cpp
virtual void HandlePathDied(
    llarp::path::Path_ptr path
) override
```

a path died now what? 

**Reimplements**: [llarp::path::PathSet::HandlePathDied](Classes/structllarp_1_1path_1_1PathSet.md#function-handlepathdied)


### function CheckPathDead

```cpp
bool CheckPathDead(
    path::Path_ptr p,
    llarp_time_t dlt
)
```


### function GetHopsForBuild

```cpp
virtual std::optional< std::vector< RouterContact > > GetHopsForBuild() override
```


**Reimplements**: [llarp::path::Builder::GetHopsForBuild](Classes/structllarp_1_1path_1_1Builder.md#function-gethopsforbuild)


### function ShouldBuildMore

```cpp
virtual bool ShouldBuildMore(
    llarp_time_t now
) const override
```

return true if we should build another path 

**Reimplements**: [llarp::path::PathSet::ShouldBuildMore](Classes/structllarp_1_1path_1_1PathSet.md#function-shouldbuildmore)


### function HandlePathBuilt

```cpp
virtual void HandlePathBuilt(
    llarp::path::Path_ptr p
) override
```


**Reimplements**: [llarp::path::Builder::HandlePathBuilt](Classes/structllarp_1_1path_1_1Builder.md#function-handlepathbuilt)


### function QueueUpstreamTraffic

```cpp
bool QueueUpstreamTraffic(
    llarp::net::IPPacket pkt,
    const size_t packSize,
    service::ProtocolType t
)
```


### function FlushUpstream

```cpp
bool FlushUpstream()
```

flush upstream to exit via paths 

### function FlushDownstream

```cpp
void FlushDownstream()
```

flush downstream to user via tun 

### function GetRoles

```cpp
inline virtual path::PathRole GetRoles() const override
```

get roles for this path builder 

**Reimplements**: [llarp::path::Builder::GetRoles](Classes/structllarp_1_1path_1_1Builder.md#function-getroles)


### function Stop

```cpp
virtual bool Stop() override
```

send close and stop session 

**Reimplements**: [llarp::path::PathSet::Stop](Classes/structllarp_1_1path_1_1PathSet.md#function-stop)


### function IsReady

```cpp
bool IsReady() const
```


### function Endpoint

```cpp
inline const llarp::RouterID Endpoint() const
```


### function CurrentPath

```cpp
inline std::optional< PathID_t > CurrentPath() const
```


### function IsExpired

```cpp
bool IsExpired(
    llarp_time_t now
) const
```


### function LoadIdentityFromFile

```cpp
bool LoadIdentityFromFile(
    const char * fname
)
```


### function AddReadyHook

```cpp
void AddReadyHook(
    SessionReadyFunc func
)
```


## Protected Functions Documentation

### function PopulateRequest

```cpp
virtual void PopulateRequest(
    llarp::routing::ObtainExitMessage & msg
) const =0
```


**Reimplemented by**: [llarp::exit::ExitSession::PopulateRequest](Classes/structllarp_1_1exit_1_1ExitSession.md#function-populaterequest), [llarp::exit::SNodeSession::PopulateRequest](Classes/structllarp_1_1exit_1_1SNodeSession.md#function-populaterequest)


### function HandleTrafficDrop

```cpp
bool HandleTrafficDrop(
    llarp::path::Path_ptr p,
    const llarp::PathID_t & path,
    uint64_t s
)
```


### function HandleGotExit

```cpp
bool HandleGotExit(
    llarp::path::Path_ptr p,
    llarp_time_t b
)
```


### function HandleTraffic

```cpp
bool HandleTraffic(
    llarp::path::Path_ptr p,
    const llarp_buffer_t & buf,
    uint64_t seqno,
    service::ProtocolType t
)
```


## Public Attributes Documentation

### variable MaxUpstreamQueueLength

```cpp
static constexpr size_t MaxUpstreamQueueLength = 256;
```


## Protected Attributes Documentation

### variable m_ExitRouter

```cpp
llarp::RouterID m_ExitRouter;
```


### variable m_ExitIdentity

```cpp
llarp::SecretKey m_ExitIdentity;
```


### variable m_WritePacket

```cpp
std::function< bool(const llarp_buffer_t &)> m_WritePacket;
```


-------------------------------

Updated on 2023-01-31 at 18:48:37 +0000